'' =================================================================================================
''
''   File....... jm_animation_demo.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2012 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 13 OCT 2012
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
' _xinfreq = 6_250_000                                          ' use 6.25MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1  = 31                                                     ' programming / terminal
  TX1  = 30
  
  SDA  = 29                                                     ' eeprom / i2c
  SCL  = 28

  LED8 = 23
  LED7 = 22
  LED6 = 21
  LED5 = 20
  LED4 = 19
  LED3 = 18
  LED2 = 17
  LED1 = 16
  

con 

  #(-1), IS_ON, IS_OFF

  #0, LSBFIRST, MSBFIRST
  

con


obj

  leds : "jm_pwm8"
 

var

  long  stack1[32]
  long  stack2[32]

  byte  work1[8]                                                ' workspace for animatons
  byte  work2[8]
  
  byte  final[8]                                                ' final output


pub main | t, idx

  leds.start(8, LED1)                                           ' start pwm

  ' start animations
  
  cognew(undulate, @stack1)
'  cognew(zig_zag(4, 150), @stack2)
  cognew(ping_pong(4, 100), @stack2)

  ' blend/prioritize as required

  t := cnt
  repeat
    repeat idx from 0 to 3                                      ' get undulation
      final[idx] := work1[idx]
      
    repeat idx from 4 to 7                                      ' get ping-pong
      final[idx] := work2[idx-4]

    bytemove(leds.address, @final, 8)                           ' update dimmer
      
    waitcnt(t += MS_001)



con

  { ----------------------- }
  {                         }
  {   A N I M A T I O N S   }
  {                         }
  { ----------------------- }


pri undulate | t, angle, idx, phase, bright                     ' launch with cognew                                           ' launch with cognew

  t := cnt
  repeat
    repeat angle from 0 to 359
      repeat idx from 0 to 3
        phase := (angle + (90 * idx)) // 360                    ' offset leds
        bright := (sin(fix_deg(phase )) ~> 9) + 128             ' 0 to 256
        bright := (bright * 9 / 10) + 25 <# 255                 ' 25 to 255                       
        work1[idx] := bright
        
      waitcnt(t += (5 * MS_001))                                ' constant loop time


pri zig_zag(n, ms) | idx                                        ' launch with cognew

'' Zig-zag n channels
'' -- led delay in ms

  repeat
    repeat idx from 0 to n-1                                    ' zip on                            
      work2[idx] := $FF
      pause(ms)

    repeat idx from n-1 to 0                                    ' zip off
      work2[idx] := $00
      pause(ms)
 

pri ping_pong(n, ms) | idx                                      ' launch with cognew

'' Ping-Pong n channels
'' -- led delay in ms

  repeat
    repeat idx from 0 to n-2                                    ' ping                           
      work2[idx] := $FF
      pause(ms)
      work2[idx] := $00

    repeat idx from n-1 to 1                                    ' pong
      work2[idx] := $FF
      pause(ms)
      work2[idx] := $00
 

con

  { --------------- }
  {                 }
  {   B A S I C S   }
  {                 }
  { --------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

con

  { --------------------------- }
  {                             } 
  {   M A T H   S U P P O R T   }
  {                             }  
  { --------------------------- }
  
  
pub fix_deg(degrees)

'' Converts degrees (0 to 360) to 13-bit value for cos and sin

  return (degrees * 1024) / 45
  
  
pub cos(x)

'' Cosine of the angle x
'' -- 0 to 360 degrees == $0000 to $2000
'' -- returns -65536 (-1) to 65536 (+1)

  return sin(x + $0800)
  

pub sin(x) | value

'' Sine of the angle x
'' -- 0 to 360 degrees == $0000 to $2000
'' -- returns -65536 (-1) to 65536 (+1)


  if ((x & $0FFF) == $0800)
    value := $1_0000
  elseif (x & $0800)
    value := word[$E000][-x & $7FF]
  else
    value := word[$E000][x & $7FF]
    
  if (x & $1000)
    value := -value

  return value
    

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}